@@ -25,7 +25,6 @@
 #include <config.h>
 #include "eel-background.h"
 
-#include "eel-background-style.h"
 #include "eel-gdk-extensions.h"
 #include "eel-gdk-pixbuf-extensions.h"
 #include "eel-glib-extensions.h"
@@ -33,6 +32,7 @@
 #include "eel-gtk-macros.h"
 #include "eel-lib-self-check-functions.h"
 #include "eel-string.h"
+#include <gtk/gtkprivate.h>
 #include <gtk/gtkselection.h>
 #include <gtk/gtksignal.h>
 #include <libart_lgpl/art_rgb.h>
@@ -169,6 +169,7 @@ eel_background_init (gpointer object, gpointer klass)
 
 	background->details = g_new0 (EelBackgroundDetails, 1);
 	background->details->constant_size = FALSE;
+	background->details->is_solid_color = TRUE;
 }
 
 /* The safe way to clear an image from a background is:
@@ -634,13 +635,29 @@ static void
 eel_background_ensure_realized (EelBackground *background, GdkWindow *window,
 				int entire_width, int entire_height)
 {
+	GdkColor color;
 	int pixmap_width, pixmap_height;
 	char *start_color_spec;
  	GdkPixmap *pixmap;
 	GdkGC *gc;
+	GtkWidget *widget;
+	GtkStyle *style;
+
+	/* Try to parse the color spec.  If we fail, default to the style's color */
 
 	start_color_spec = eel_gradient_get_start_color_spec (background->details->color);
-	eel_gdk_color_parse_with_white_default (start_color_spec, &background->details->background_color);
+
+	if (start_color_spec && eel_gdk_color_parse (start_color_spec, &color))
+		background->details->background_color = color;
+	else {
+		/* Get the widget to which the window belongs and its style as well */
+		gdk_window_get_user_data (window, (void **) &widget);
+		g_assert (widget != NULL);
+
+		style = gtk_widget_get_style (widget);
+		background->details->background_color = style->bg[GTK_STATE_NORMAL];
+	}
+
 	g_free (start_color_spec);
 
 	/* If the pixmap doesn't change with the window size, never update
@@ -697,6 +714,72 @@ eel_background_get_pixmap_and_color (EelBackground *background,
 	return NULL;
 }
 
+void
+eel_background_expose (GtkWidget                   *widget,
+		       GdkEventExpose              *event)
+{
+	GdkColor color;
+	int window_width;
+	int window_height;
+	gboolean changes_with_size;
+	GdkPixmap *pixmap;
+	GdkGC *gc;
+	GdkGCValues gc_values;
+	GdkGCValuesMask value_mask;
+
+	EelBackground *background;
+	
+	if (event->window != widget->window) {
+		return;
+	}
+	
+	background = eel_get_widget_background (widget);
+
+	gdk_drawable_get_size (widget->window, &window_width, &window_height);
+	
+	pixmap = eel_background_get_pixmap_and_color (background,
+						      widget->window,
+						      window_width,
+						      window_height,
+						      &color,
+						      &changes_with_size);
+	
+        if (!changes_with_size) {
+                /* The background was already drawn by X, since we set
+                 * the GdkWindow background/back_pixmap.
+                 * No need to draw it again. */
+                if (pixmap) {
+                        g_object_unref (pixmap);
+                }
+                return;
+        }
+ 
+	if (pixmap) {
+		gc_values.tile = pixmap;
+		gc_values.ts_x_origin = 0;
+		gc_values.ts_y_origin = 0;
+		gc_values.fill = GDK_TILED;
+		value_mask = GDK_GC_FILL | GDK_GC_TILE | GDK_GC_TS_X_ORIGIN | GDK_GC_TS_Y_ORIGIN;
+	} else {
+		gdk_rgb_find_color (gtk_widget_get_colormap (widget), &color);
+		gc_values.foreground = color;
+		gc_values.fill = GDK_SOLID;
+		value_mask = GDK_GC_FILL | GDK_GC_FOREGROUND;
+	}
+	
+	gc = gdk_gc_new_with_values (widget->window, &gc_values, value_mask);
+	
+	gdk_gc_set_clip_rectangle (gc, &event->area);
+
+	gdk_draw_rectangle (widget->window, gc, TRUE, 0, 0, window_width, window_height);
+	
+	g_object_unref (gc);
+	
+	if (pixmap) {
+		g_object_unref (pixmap);
+	}
+}
+
 void
 eel_background_pre_draw (EelBackground *background, int entire_width, int entire_height)
 {
@@ -1105,48 +1188,6 @@ eel_background_receive_dropped_background_image (EelBackground *background,
 	eel_background_set_image_uri_and_color (background, image_uri, NULL);
 }
 
-static void
-eel_background_set_style_font_from_default (GtkStyle *style,
-					    GtkSettings *settings)
-{
-	char *fontname;
-
-	fontname = NULL;
-	
-	g_object_get (settings,
-		      "gtk-font-name", &fontname,
-		      NULL);
-
-	if (fontname != NULL) {
-		if (style->font_desc) {
-			pango_font_description_free (style->font_desc);
-		}
-		
-		style->font_desc = pango_font_description_from_string (fontname);
-		g_free (fontname);
-	}
-}
-
-
-static void
-eel_background_set_widget_style (EelBackground *background,
-				 GtkWidget *widget)
-{
-	GtkStyle *style;
-	
-	g_return_if_fail (EEL_IS_BACKGROUND (background));
-	g_return_if_fail (GTK_IS_WIDGET (widget));
-	
-	style = GTK_STYLE (eel_background_style_new (gtk_widget_get_style (widget),
-						     background));
-	
-	/* Bad hack to make font changing work: */
-	eel_background_set_style_font_from_default (style, gtk_widget_get_settings (widget));
-	
-	gtk_widget_set_style (widget, style);
-	g_object_unref (style);
-}
-
 /**
  * eel_background_is_set:
  * 
@@ -1254,6 +1295,44 @@ render_background_callback (GnomeCanvas *canvas, GnomeCanvasBuf *buffer)
 	g_signal_stop_emission_by_name (canvas, "render_background");
 }
 
+static void
+eel_background_set_up_widget (EelBackground *background, GtkWidget *widget)
+{
+	GtkStyle *style;
+	GdkPixmap *pixmap;
+	GdkColor color;
+	int window_width;
+	int window_height;
+	gboolean changes_with_size;
+
+	if (!GTK_WIDGET_REALIZED (widget)) {
+		return;
+	}
+
+	gdk_drawable_get_size (widget->window, &window_width, &window_height);
+	
+	pixmap = eel_background_get_pixmap_and_color (background,
+						      widget->window,
+						      window_width,
+						      window_height,
+						      &color, 
+						      &changes_with_size);
+
+	style = gtk_widget_get_style (widget);
+	
+	gdk_rgb_find_color (style->colormap, &color);
+	
+	if (pixmap && !changes_with_size) {
+		gdk_window_set_back_pixmap (widget->window, pixmap, FALSE);
+	} else {
+		gdk_window_set_background (widget->window, &color);
+	}
+
+	if (pixmap) {
+		g_object_unref (pixmap);
+	}
+}
+
 static void
 eel_background_set_up_canvas (GtkWidget *widget)
 {
@@ -1278,37 +1357,22 @@ static void
 eel_widget_background_changed (GtkWidget *widget, EelBackground *background)
 {
 	eel_background_unrealize (background);
-	eel_background_set_widget_style (background, widget);
+	eel_background_set_up_widget (background, widget);
 	eel_background_set_up_canvas (widget);
 
 	gtk_widget_queue_draw (widget);
 }
 
-/* Bad hack to make font changes work for EelBackgrounds */
-static void
-eel_widget_font_changed (GtkWidget *widget,
-			 GParamSpec *pspec,
-			 GtkSettings *settings)
-{
-	EelBackground *background;
-	GtkStyle *style;
-
-	background = eel_get_widget_background (widget);
-
-	style = gtk_style_copy (gtk_widget_get_style (widget));
-	
-	eel_background_set_style_font_from_default (style, settings);
-		
-	gtk_widget_set_style (widget, style);
-	g_object_unref (style);
-}
-
 /* Gets the background attached to a widget.
 
    If the widget doesn't already have a EelBackground object,
    this will create one. To change the widget's background, you can
    just call eel_background methods on the widget.
 
+   If the widget is a canvas, nothing more needs to be done.  For
+   normal widgets, you need to call eel_background_expose() from your
+   expose handler to draw the background.
+
    Later, we might want a call to find out if we already have a background,
    or a way to share the same background among multiple widgets; both would
    be straightforward.
@@ -1318,7 +1382,6 @@ eel_get_widget_background (GtkWidget *widget)
 {
 	gpointer data;
 	EelBackground *background;
-	GtkSettings *settings;
 
 	g_return_val_if_fail (GTK_IS_WIDGET (widget), NULL);
 
@@ -1341,9 +1404,6 @@ eel_get_widget_background (GtkWidget *widget)
 				 G_CALLBACK (eel_widget_background_changed), widget, G_CONNECT_SWAPPED);
 	eel_widget_background_changed (widget, background);
 
-	settings = gtk_widget_get_settings (widget);
-	g_signal_connect_object (settings, "notify::gtk-font-name",
-				 G_CALLBACK (eel_widget_font_changed), widget, G_CONNECT_SWAPPED);
 	return background;
 }
 
